/*
	DCX2CIFC.C - DCX-AT200 'C' language interface module
   Copyright Precision Micro Control Corp. 1988-1994, All rights reserved
   Revision: 1.1a
   Date Last Modified: 9/6/94
   Programmer: Bruce Robinson
   Compiler: Microsoft C version 7.0

	Abstract: This module contains functions written in the C programming
   language for interfacing with DCX boards.  The code in this module can
   be compiled and linked into the end users application program.

	Revision History
   1.0a - Initial release (created from DCX series 100 version: DCXCIFC.C)
	1.1a - Fixed comment in front of AL command code define.
		  - Fixed comment after OA command code define.
		  - Fixed define for MC260 module type.
	     - Changed the names of Amplifier Fault flags to MTRAFE and MTRAFT.
		  - Added return values to functions dcxcmd() and dcxvcmd().
		  - Re-ordered command code define statements to match user's manual
		    listing.
		  - Created DCX2CMAC.H file for macros supplied with interface library.
*/

#define DCXLIB

#include <stdio.h>
#include <dos.h>
#include "dcx2cifc.h"


/* Declarations of buffers for receiving replies. These buffers are used
	in DCX2CMAC.H */
struct rpylngfmt lngrpybuf[MOTORS_PER_BOARD];
struct rpyfltfmt fltrpybuf[MOTORS_PER_BOARD];
struct rpydblfmt dblrpybuf[MOTORS_PER_BOARD];


unsigned char byt;			/* Byte used during reads to far memory. */
unsigned char far *bytptr = &byt;		/* Note FAR prefix of this pointer. */


/*
 dcxvcmd() - Send variable number of commands to a DCX controller using
             binary interface. This function has a parameter specifiying
				 the number of bytes to be passed to the DCX board. Another
				 function in this library, dcxcmd(), has a similar format but
				 has a parameter that specifies the number of commands to be
				 passed to the DCX. That function assumes each command has a
				 4 byte parameter. This does not support the added capability
				 of the series 200 DCX board to accept commands with no
				 parameter or commands with floating point formatted parameters.

 Function parameters:
	board = DCX board number (0-15).
	count = number of bytes in DCX commands.
	cmd = marks first DCX command, there can be up to 40 commands
			passed to function.  Commands will be passed on stack as a
			16 bit value containing command code and axis number followed
			by an optional 32 or 64 bit command parameter value.

 Function returns number of bytes sent to board.

 Example:
		long position;
		dcxcmd(0, 18, AXIS1 + MA, position, AXIS1 + WS, 100L, TP, 0L);

  After calling this function, the reply to the end TP command can be
  read using the dcxrpy() function.
*/
int dcxvcmd(board,count,cmd,...)
	int board,count,cmd;
{
	int far *cmmdptr;

	cmmdptr = &cmd;	/* Set pointer to commands on PC stack. */

	while(1)
	{
		/* Get present count from command block. */
		movedata(DCXSEG+(board*0x100),CMDCNT,
												FP_SEG(bytptr),FP_OFF(bytptr),1);
		/* If count is nonzero, last command has not been processed by DCX. */
		if (byt==0)
		{
			/* Move commands from PC stack to DCX dual port. */
			movedata(FP_SEG(cmmdptr),FP_OFF(cmmdptr),
												DCXSEG+(board*0x100),CMDBLK,count);
			/* Set reply counter to 0. */
			byt = 0;
			movedata(FP_SEG(bytptr),FP_OFF(bytptr),
												DCXSEG+(board*0x100),RPYCNT,1);
			/* Set command counter tonumber of valid bytes in command block. */
			byt = (unsigned char)count;
			movedata(FP_SEG(bytptr),FP_OFF(bytptr),
												DCXSEG+(board*0x100),CMDCNT,1);
			return(count);
		}
	}
}



/*
 dcxcmd() - Send one or more commands to a DCX controller through binary
				command buffer. This function has a parameter that specifies
				the number of commands to be passed to the DCX. It assumes
				each command has a 32 bit command parameter. Thus, it does
				not support sending commands with no parameter or commands
				with 64 bit floating point parameters.  This function has
				been replaced by the more flexible version dcxvcmd(), but
				is included in the library for backward compatibility. New
				designs should used the dcxvcmd() if possible.

 Function parameters:
	board = DCX board number (0-15).
	count = number of DCX commands.
	cmd = marks first DCX command, there can be up to 40 commands
			passed to function.  Commands will be passed on stack as a
			16 bit value containing command code and axis number followed
			by a 32 bit command parameter value.

 Function returns number of commands sent to board.

 Example:
		long position;
		dcxcmd(0, 3, AXIS1 + MA, position, AXIS1 + WS, 100L, TP, 0L);

  After calling this function, the reply to the end TP command can be
  read using the dcxrpy() function.
*/
int dcxcmd(board,count,cmd,...)
	int board,count,cmd;
{
	int far *cmmdptr;

	cmmdptr = &cmd;	/* Set pointer to commands on PC stack. */

	while(1)
	{
		/* Get present count from command block. */
		movedata(DCXSEG+(board*0x100),CMDCNT,
												FP_SEG(bytptr),FP_OFF(bytptr),1);
		/* If count is nonzero, last command has not been processed by DCX. */
		if (byt==0)
		{
			/* Move commands from PC stack to DCX dual port. */
			movedata(FP_SEG(cmmdptr),FP_OFF(cmmdptr),
												DCXSEG+(board*0x100),CMDBLK,count*6);
			/* Set reply counter to 0. */
			byt = 0;
			movedata(FP_SEG(bytptr),FP_OFF(bytptr),
												DCXSEG+(board*0x100),RPYCNT,1);
			/* Set command counter to number of valid bytes in command block. */
			byt = (unsigned char)count*6;
			movedata(FP_SEG(bytptr),FP_OFF(bytptr),
												DCXSEG+(board*0x100),CMDCNT,1);
			return(count);
		}
	}
}



/*
 dcxrpy() - Retrieve reply from DCX through binary reply buffer.
 Function parameters:
	board = DCX board number (0-15).
	count = size of reply buffer
	rplyptr = Far pointer to buffer where reply from DCX should be placed.
			passed to function.  Return values will be placed in the buffer
			as 16 bit values containing the axis number followed by either
			a 16 or 32 bit value.

 Example:
		int rpybuf[24];
		dcxrpy(0, sizeof(rpybuf), (int far *)rpybuf);
*/
int dcxrpy(board,count,rplyptr)
	int board, count;
	int far *rplyptr;
{

	while(1)
	{
		/* Get present count from command block. */
		movedata(DCXSEG+(board*0x100),CMDCNT,
												FP_SEG(bytptr),FP_OFF(bytptr),1);
		/* If count is nonzero, last command has not been processed by DCX. */
		if (byt==0)
		{
			/* Get reply counter. */
			movedata(DCXSEG+(board*0x100),RPYCNT,
												FP_SEG(bytptr),FP_OFF(bytptr),1);
			if (byt < count)
				count	= byt;
			/* Move reply from DCX dual port to reply buffer. */
			movedata(DCXSEG+(board*0x100),RPYBLK,
												FP_SEG(rplyptr),FP_OFF(rplyptr),count);
			return(count);
		}
	}
}


/*
 dcxrdy() - Test if DCX is ready for next command.
				ie. Is command buffer empty?
 Function parameters:
	board = DCX board number (0-15).

 Returns:
	1 if ready
	0 if busy

*/
int dcxrdy(board)
	int board;
{
	/* Get present count from command block. */
	movedata(DCXSEG+(board*0x100),CMDCNT,
												FP_SEG(bytptr),FP_OFF(bytptr),1);
	if (byt)
		return(0);
	else
		return(1);
}



/*
 putdcx() - Pass ascii character to DCX board through mailbox.
 This command can be used to implement programs which send commands
 to the DCX in ascii format.
 Returns 0 if successful, ascii character if DCX busy.
*/
putdcx(board,dbyte)
	int board,dbyte;
{
	/* Get present value at Data Input mailbox. */
	movedata(DCXSEG+(board*0x100),DATAIN,
												FP_SEG(bytptr),FP_OFF(bytptr),1);
	/* If contents of mailbox is nonzero, last byte has not been read
			by DCX. */
	if (byt==0)
	{
		byt = dbyte;
		/* Write new data to DCX. */
		movedata(FP_SEG(bytptr),FP_OFF(bytptr),
												DCXSEG+(board*0x100),DATAIN,1);
		return(0);
	}
	else
		return(dbyte);
}


/*
 getdcx() - Get ascii character from DCX through mailbox.
 Returns ascii character or 0 if not ready
*/
getdcx(board)
	int board;
{
	char temp;

	/* Check for DCX sending data to Data Out mailbox. */
	movedata(DCXSEG+(board*0x100),DATAOUT,
												FP_SEG(bytptr),FP_OFF(bytptr),1);
	temp = byt;
	if (temp)
	{
		byt = 0;
		/* Write a zero to Data In mail box to signal that we have
			received data and are ready for new data. */
		movedata(FP_SEG(bytptr),FP_OFF(bytptr),
												DCXSEG+(board*0x100),DATAOUT,1);
	}
	return(temp);
}


/* Note:
    The movedata() function used in this program is available in the
    Microsoft C libraries.  It is used to move a specifed number of bytes
    from anywhere in the PC's memory to any other place.  It has the
    following format:

      movedata(src segment, src offset, dest segment, dest offset, count)

    If this program is being compiled under another brand, this function
    will have to be created or replaced with the equivalent.
*/


/* End of DCX2CIFC.C */
