/*
    DCX-AT200 Control Program
    Revision: 1.5a
    Date Last Modified: 5/7/96
    Programmer: Bruce Robinson                                                       
    Precision MicroControl Corporation
    Copyright (c) 1989-1996, All rights reserved                                       

    Abstract:
       This file contains a program for implementing a PC to DCX-AT200
    interface.  When running, this program will send any characters typed
    at the keyboard to a DCX board installed in the computer.  The program
    will also display any charaters received from the board. This program
    defaults to communication with DCX board number 0. This is the board
    whose rotary switch is set to 0.
       Another feature of this program is the ability to have a file
    containing DCX commands to be sent to the DCX board. In order
    do this, place the name of a text file containg DCX commands on the
    DOS command line when invoking this program.

    Example: DCX2CTRL commandfile

	 There are a number of options the user can specify when invoking this
    program. The following option specifiers are supported:

		/Bn	Causes program to access board n (default = 0).

		/T		Causes the DCX Control program to terminate itself after
				sending the file of DCX commands. This option has no effect
				if a DCX command file was not specified on the DOS command line.

		/P		Causes the DCX Control program to Pause after sending a
				DCX command from a file until a key is pressed.


    Example: DCX2CTRL commandfile /T
*/

/* To build program under Microsoft C use:  CL DCX2CTRL.C	*/

/*
	Revision History
	1.0a	Initial release. Created from DCXCNTRL program, revision 1.4a.
			This version of the program expects the controller board to
			echo each character sent to it. This allows better feedback
			from the board in special cases such as when the board is
			busy executing a command.

	1.1a  Modified program to locally display characters sent to DCX board
			when downloading a file. This results in a cleaner display during
			file downloads if Echo is oFf.

	1.5a  Added exit conditions for program matching modified DCX-AT200
	      firmware revision 1.5a. New firmware will not clear command
			interpreter status until final prompt character has been placed
			in output mailbox. This provides a definite set of conditions
			that indicate that the controller has completed the last command
			and all characters have been sent. This firmware also sets the
			controller's command interpreter status to non-zero before
			removing a carriage return from the input mailbox. This avoids
			race conditions between the controller and the DCX2CTRL program
			trying to send the next command from a file. Because prompt
			character is now guranteed to precede the text of a command line,
			changed program to not do local echoeing of file download text.
			All program display will come from echo of controller board. If
			no display is required, echo can be turned off on controller.

*/

#include <dos.h>
#include <stdio.h>
#include <string.h>

#define DATAIN 0x800			/* Memory offset for data input mailbox. */
#define DATAOUT 0x804		/* Memory offset for data output mailbox. */
#define CMDSTAT 0x808		/* Memory offset for command status. */

unsigned int dcxseg = 0xD000;	/* Memory sement where DCX dual port ram is. */

unsigned char byt;			/* Byte used during reads to far memory. */
unsigned char far *bytptr;		/* Note FAR prefix of this pointer. */

char cmdline[256];

main(argc,argv)
int argc;
char *argv[];
{
	int argi;			/* Command line argument index. */
	char option;		/* Command line options. */
	char key = 0;		/* Last character from keyboard or file. */
	char chrin;			/* Last character received from DCX output mailbox. */
	int idx;
	int terminate = 0;	/* Flag indicating program should terminate after
									loading DCX command file. */
	int pause = 0;			/* Flag indicating program should pause after
									sending DCX command from file until a key is
									pressed.*/
	int wait = 0;
	int dcxbusy;
	FILE *cmdfile = 0;
	char bdnumchar;

	printf("DCX-AT200 Control Program   Revision 1.5a\r\n");
	printf("Copyright (c) Precision MicroControl Corporation 1989-1996\r\n");
	printf("All rights reserved\r\n");

	bytptr = &byt;			/* Initialize far pointer. */

												/* Parse command line */
	for (argi = 1; argi < argc; argi++)
	{
		if (*argv[argi] == '/')			/* Test for command option having '/' */
		{
			option = toupper(*(argv[argi]+1));
											/* 'Bn' option will cause program to access
												selected DCX board. */
			if (option=='B')
			{
				bdnumchar = toupper(*(argv[argi]+2));
				if ((bdnumchar >= '0') && (bdnumchar <= '9'))
					dcxseg += ((bdnumchar - '0') << 8);
				else if ((bdnumchar >= 'A') && (bdnumchar <= 'F'))
					dcxseg += ((bdnumchar - 'A') << 8);
				else
				{
					printf("bad board number\r\n");
					exit(0);
				}
				printf("board number %c\r\n",bdnumchar);
			}
											/* 'T' option will cause program to 
												terminate after loading DCX command
												file. */
			if (option=='T')
				terminate = 1;
											/* 'P' option causes pause after each
												command line until a key is pressed. */
			else if (option=='P')
				pause = 1;
		}
		else						/* If not command option, should be file name. */
		{
									/* Try to open file for reading. */
			if (cmdfile = fopen(argv[argi],"r"))
				printf("command file : %s\r\n", argv[argi]);
			else
			{
				/* If unable to open file, print message and exit. */
				printf("can't open : %s\r\n", argv[argi]);
				exit(0);
			}
			idx = -1;
		}
	}

	if (cmdfile==0)
		printf("press <CNTRL> C to exit\r\n");

	/* If DCX command interpreter busy, send ascii CANCEL code to clear. */ 
	if (dcxbusy = getdcxbyt(CMDSTAT))
		key = 0x18;

	if (pause)	/* Print message and set wait if pause enabled. */
	{
		wait = 1;
		printf("Ready, press any key to start...\r\n");
	}

	/* Main loop. Breaking out of this loop requires that the terminate
		DOS command line parameter was set, the file has been sent, the
		controller has completed the last command, and the prompt character
		(if any) sitting in the DCX output mailbox is received */
	while(!terminate || cmdfile || getdcxbyt(CMDSTAT) || getdcxbyt(DATAOUT))
	{
		/* If contents of key is 0, get next character from command
			file or from the keyboard. */
		if (!key)
		{
			/* If sending command file, get next charater from file. */
			if (cmdfile)
			{
				/* Wait for data input mailbox to be empty. This allows DCX
					enough time to set busy if it has recieved a full command. */
				if (getdcxbyt(DATAIN)==0)
				{

					/* Index less than zero indicates need to read next line. */
					if (idx<0)
					{
						if (fgets(cmdline,sizeof(cmdline),cmdfile))
							idx = 0;		/* Initialize string index to beginning. */
						else
						{
							if (fclose(cmdfile) != 0)
								printf("error closing file\r\n");
							cmdfile = 0;
							if (!terminate)
							{
								printf("End of command file\r\n");
								printf("press <CNTRL> C to exit\r\n");
							}
						}
					}

					/* Test whether previous command	has finished */
					if (dcxbusy)
					{
						dcxbusy = getdcxbyt(CMDSTAT);
						if (!dcxbusy && pause)	/* Set wait flag if pause enabled. */
						{
							wait = 1;
							printf("Ready...\r\n");
						}
					}

					if (wait && kbhit())
					{
						getch();
						wait = 0;
					}

					if (!dcxbusy && !wait && (idx>=0))
					{
						key = cmdline[idx];	/* Store next character in key variable. */

													/* Test for '\' followed immediatly by
														carriage return. */
						if ((key=='\\') &&
									((cmdline[idx+1]==0xD) || (cmdline[idx+1]==0xA) ||
									 	(cmdline[idx+1]==0)))
						{
							key = 0;
							idx = -1;
						}
													/* Test for end of command line. */
						else if ((key==' ') || (key=='\t') || (key == ';') ||
										(key == '\n') || (key == '\r'))
						{
							if (idx)			/* Is this an empty line? */
							{
								key = '\r';		/* End of command line. */
								dcxbusy = 1;	/* Set busy flag to prevent sending
														next command until this one
														completed. */
							}
							else
								key = 0;		/* Reject empty lines. */ 
							idx = -1;	/* At end of line reset index. */
						}

						else
							idx++;
					}
				}
			}

			/* If not sending command file, check for keypress. */
			else if (kbhit())
				key = getch();		/* Store character in key. */
		}

		/* If key is nonzero, it contains next code to be sent to DCX board. */
		if (key)
		{
			/* Get present value at Data Input mailbox. If contents of mailbox
				 is nonzero, last byte has not been read by DCX. */
			if (getdcxbyt(DATAIN)==0)
			{
				putdcxbyt(DATAIN,key);		/* Write new data to DCX. */
				key = 0;
			}
		}

		/* Check for DCX sending data to Data Output mailbox. */
		if ((chrin=getdcxbyt(DATAOUT)) != 0)
		{
			putch(chrin);		/* Send data to screen for user display. */

			if (chrin == '\r')			/* Add linefeeds to carriage return */
			{
				putch('\n');
			}
			else if (chrin == '\b')		/* Clear character for back space */
			{
				putch(' ');
				putch('\b');
			}

			/* Write a zero to Data In mail box to signal that we have
				received data and are ready for new data. */
			putdcxbyt(DATAOUT,0);
		}

	}

}


int getdcxbyt(loc)
	int loc;
{
	movedata(dcxseg,loc,FP_SEG(bytptr),FP_OFF(bytptr),1);
	return(byt);
}


putdcxbyt(loc,val)
	int loc,val;
{
	byt = val;
	movedata(FP_SEG(bytptr),FP_OFF(bytptr),dcxseg,loc,1);
}


/* Note:
    The movedata() function used in this program is available in the
    Microsoft C libraries.  It is used to move a specifed number of bytes
    from anywhere in the PC's memory to any other place.  It has the
    following format:

      movedata(src segment, src offset, dest segment, dest offset, count)

    If this program is being compiled under another brand, this function
    will have to be created or replaced with the equivalent.
*/
