{**************************************************************************
	Copyright (c) Precision Micro Control Corp. 1992-1994
	All rights reserved
**************************************************************************}

unit DCX2PIFC;

{
	DCXPIFC.PAS - DCX-AT200 Pascal Interface Unit
	Revision: 1.0a
	Date last modiifed: 9/6/94
	Programmer: Bruce Robinson


	Abstract: This unit contains Pascal procedures and functions for
	interfacing with the DCX-AT200 Motion Controller. To use the items
	in this unit, place a "uses" statement in the program block.
	Example Program:

		program demo;
		uses DCXPIFC;
		begin
			if (dcxbusy(0)) then
				writeln('DCX BUSY!');
		end.

	Revision History:
	1.0a	9/6/94  Initial release

}

interface

const

{ DCX dual port memory equates. }
	DCXSEG = $D000;		{ Memory sement where DCX dual port ram is. }
	DATAOUT = $804;      { Memory offset for data out mailbox. }
	DATAIN = $800;       { Memory offset for data in mailbox. }
	CMDSTAT = $808;		{ Memory offset for command status. }
	MODLOC = $80A;			{ Memory offset for motor module locations. }
	MODTYP = $812;			{ Memory offset for array of module types. }
	CMDCNT = $900;			{ Memory offset for command count. }
	CMDBLK = $901;			{ Memory offset for command block. }
	RPYCNT = $A00;			{ Memory offset for reply count. }
	RPYBLK = $A01;			{ Memory offset for reply block. }

{ Module ID codes }
	MC200 =  0;		{ Advanced Servo }
	MC260 =  1;		{ Advanced Stepper }
	MC100 =  5;		{ Servo with analog signal output }
	MC110 =  4;		{ Servo with motor drive output }
	MC120 =  4;		{ Servo with PWM output }
	MC150 =  6;		{ Stepper motor }
	MC160 =  7;		{ Stepper motor with encoder }
	MC400 =  8;		{ Digital I/O module }
	MC500 =  12;	{ Analog I/O module }
	MF310 =  9;		{ GPIB interface }
	MF300 =  10;	{ RS232 interface }
	NOMOD	=	15;	{ No module installed }

{ Axis values for parameters to dcxcmd() function. } 
	AXIS1	=	$100;
	AXIS2	=	$200;
	AXIS3	=	$300;
	AXIS4	=	$400;
	AXIS5 =	$500;
	AXIS6	=	$600;

{ Modifier flags for axis bytes in the dcxcmd() and dcxvcmd() function
	parameters }
	INDPARFLAG	=	$8000;	{ Indirect parameter flag }
	NOPARFLAG	=	$6000;	{ No parameter flag }
	SGLPARFLAG	=	$4000;	{ Single parameter flag }
	DBLPARFLAG	=	$2000;	{ Double parameter flag }

{ Number of bytes in binary DCX commands (depends on type of parameter) }
	CMDSZ		=	2;				{ no parameter }
	LNGCMDSZ	=	6;				{ with long parameter }
	SGLCMDSZ	=	6;				{ with single parameter }
	DBLCMDSZ	=	10;			{ with double parameter }

{ Parameter values for Tell commands }
	LNGRPY	=	0;       { Tell command will have long integer reply format }
	DBLRPY	=	1;       { Tell command will have double reply format }
	SGLRPY	=	2;			{ Tell command will have single reply format }

{ Motor data offsets }
	STAT		=	$00;	{ Motor status }
	POSCNT	=	$04;	{ Position Count (encoder or steps) }
	OPTCNT	=	$08;	{ Optimal Count (encoder or steps) }
	IDXCNT	=	$0C;	{ Index Count (encoder or steps) }
	AUXSTAT	=	$10;	{ Auxiliary Motor Status }
	POSITION =	$14;	{ Position in user units }
	TARGET	=	$1C;	{ Tareget in user units }
	OPTIMAL	=	$24;	{ Optimal position in user units }
	BRKPOS	=	$2C;	{ Break position in user units }
	DEADBAND	=	$34;	{ Deadband in user units }
	MAXERROR	=	$3C;	{ Maximum following error in user units }
	LOLIMIT	=	$44;	{ Low limit of movement in user units }
	HILIMIT	=	$4C;	{ High limit of movement in user units } 
	SCALE		=	$54;	{ User Scale }
	ZERO		=	$5C;	{ User Zero }
	OFFSET	=	$64;	{ User Offset }
	RATE		=	$6C;	{ User Rate conversion }
	OUTCONST	=	$74;	{ User output Konstant }
	PGMVEL	=	$7C;	{ Programmed velocity in user units } 
	PGMACC	=	$84;	{ Programmed acceleration in user units } 
	PGMDEC	=	$8C;	{ Programmed deceleration in user units } 
	MINVEL	=	$94;	{ Minimum velocity in user units }
	CURVEL	=	$9C;	{ Current velocity in user units }
	KVEL		=	$A4;	{ Velocity gain, in user distance units }
	KACC		=	$A8;	{ Acceleration gain, in user distance units }
	KDEC		=	$AC;	{ Deceleration gain, in user distance units }
	VELOVRD	=	$B0;	{ Velocity override }
	MAXTRQ	=	$B4;	{ Torque limit }
	KPOS		=	$B8;	{ Proportional gain }
	KDER		=	$BC;	{ Derivative gain }
	KINT		=	$C0;	{ Integral gain }
	ILIM		=	$C4;	{ Integration limit }
	ADCIN1	=	$C8;	{ Analog input 1 on module }
	ADCIN2	=	$CC;	{ Analog input 2 on module }
	JGAIN		=	$D0;	{ Jog proportional Gain }
	JOFFSET	=	$D4;	{ Jog Offset }
	JBAND		=	$D8;	{ Jog deadBand }
	JOGACC	=	$DC;	{ Jog acceleration in user units } 
	JOGMINVEL =	$E4;	{ Jog minimum velocity in user units }
	WAITSTOP	=	$EC;	{ Wait stopped timer }
	WAITTRGT	=	$EE;	{ Wait target timer }
	SFREQ		=	$F0;	{ Sampling frequency }
	MASTER	=	$F2;	{ Set to axis that is master }
	MODST		=	$F4;	{ Module status }
	AXISNUM	=	$F6;	{ Axis number 1-6 }
	MODULE	=	$F8;	{ Module position }
	MODTYPE	=	$FA;	{ Module type }


{ Motor status bit definitions }
	MTRBSY =	$01;	{ BUSY (SERVO PARAMETERS BEING UPDATAED) }
	MTRON	 =	$02;	{ MOTOR ON }
	MTRAT	 =	$04;	{ AT TARGET }
	MTRTC	 =	$08;	{ TRAJECTORY COMPLETE }
	MTRDI  =	$10;	{ DIRECTION ( 0 = POSITIVE, 1 = NEGATIVE ) }
	MTRJOG =	$20;	{ JOGGING IS ENABLED }
	MTRHMD =	$40;	{ MOTOR HOMED }
	MTRERR =	$80;	{ MOTOR ERROR }
	MTRLFI =	$0100;	{ LOOK FOR INDEX }
	MTRLFE =	$0200;	{ LOOK FOR EDGE }
	MTRBRK =	$1000;	{ BREAKPOINT REACHED }
	MTREFE =	$2000;	{ EXCEEDED MAXIMUM FOLLOWING ERROR }
	MTRAFE =	$4000;	{ AMPLIFIER FAULT ENABLED }
	MTRAFT =	$8000;	{ AMPLIFIER FAULT TRIPPED }
	MTRLPE = $10000;		{ HARD LIMIT POSITIVE ENABLED }
	MTRLPT = $20000;		{ HARD LIMIT POSITIVE TRIPPED }
	MTRLNE = $40000;		{ HARD LIMIT NEGATIVE ENABLED }
	MTRLNT = $80000;		{ HARD LIMIT NEGATIVE TRIPPED }
	MTRLHE = $100000;		{ SOFT POSITION LIMIT HIGH ENABLED }
	MTRLHT = $200000;		{ SOFT POSITION LIMIT HIGH TRIPPED }
	MTRLLE = $400000;		{ SOFT POSITION LIMIT LOW ENABLED }
	MTRLLT = $800000;		{ SOFT POSITION LIMIT LOW TRIPPED }
	MTRIDX = $1000000;	{ ENCODER INDEX / STEPPER HOME }
	MTRCRS = $2000000;	{ COARSE HOME }
	MTREE  =	$4000000;	{ AMPLIFIER FAULT }
	MTRRSV = $8000000;	{ AUXILIARY ENCODER INDEX }
	MTRLMR = $10000000;	{ LIMIT POSITIVE INPUT ACTIVE }
	MTRLML = $20000000;	{ LIMIT NEGATIVE INPUT ACTIVE }
	MTRJGR = $40000000;	{ USER INPUT 1 }
	MTRJGL = $80000000;	{ USER INPUT 2 }


{ DCX Command Codes }
{ Parameter Setup Commands }
	AG	=	226;		{ set Acceleration Feed-Forward Gain }
	AH	=	234;		{ Auxiliary encoder define Home }
	BR	=	30;		{ Baud Rate }
	CF	=	31;		{ Channel ofF }
	CH	=	66;		{ Channel High true }
	CI	=	32;		{ Channel In }
	CL	=	67;		{ Channel Low true }
	CN	=	33;		{ channel oN }
	CT	=	34;		{ Channel ouT }
	DB	=	118;		{ set DeadBand }
	DG	=	227;		{ set Deceleration Feed-Forward Gain }
	DH	=	35;		{ Define Home }
	DI	=	36;		{ DIrection }
	DS	=	117;		{ Deceleration Set }
	DT	=	197;		{ at Target Delay }
	EF	=	37;		{ Echo ofF }
	EN	=	38;		{ Echo oN }
	FC	=	64;		{ Full Current }
	FF	=	51;		{ Fail input ofF }
	FN	=	50;		{ Fail input oN }
	FR	=	39;		{ set derivative sampling period }
	HC	=	65;		{ Half Current }
	HF	=	48;		{ RS232 module Handshake ofF }
	HL	=	211;		{ set motion High Limit }
	HN	=	49;		{ RS232 module Handshake oN }
	HS	=	232;		{ High Speed }
	IL	=	40;		{ set Integration Limit }
	JA	=	56;		{ Jog Acceleration }
	JB	=	223;		{ Jog Deadband }
	JF	=	58;		{ Jog ofF }
	JG	=	220;		{ Jog Gain }
	JN	=	57;		{ Jog oN }
	JO	=	222;		{ Jog Offset }
	JV	=	55;		{ Jog Velocity }
	LF	=	54;		{ Limit switch ofF }
	LL	=	210;		{ set motion Low Limit }
	LM	=	52;		{ Limit switch Mode }
	LN	=	53;		{ Limit switch oN }
	LS	=	230;		{ Low Speed }
	MS	=	231;		{ Medium Speed }
	MV	=	196;		{ set Minimum Velocity }
	OM	=	216;		{ set Output Mode }
	PC	=	128;		{ Prompt Character }
	PH	=	115;		{ Set Phasing }
	PS	=	238;		{ Profile S-curve }
	PT	=	239;		{ Profile Trapezoidal }
	RT	=	42;		{ ReseT system }
	SA	=	43;		{ Set Acceleration }
	SD	=	44;		{ Set Derivative gain }
	SE	=	25;		{ Stop on Error }
	SF	=	62;		{ Step Full }
	SG	=	45;		{ Set Gain }
	SH	=	63;		{ Step Half }
	SI	=	46;		{ Set Integral gain }
	SQ	=	116;		{ Set torque }
	SS	=	29;		{ Set Slave ratio }
	SV	=	47;		{ Set Velocity }
	UK	=	215;		{ set User output Konstant }
	UO	=	179;		{ set User Offset }
	UR	=	177;		{ set User Rate conversion }
	US	=	175;		{ set User Scale }
	UT	=	178;		{ set User Time conversion }
	UZ	=	176;		{ set User Zero }
	VA	=	173;		{ Vector Acceleration }
	VD	=	174;		{ Vector Deceleration }
	VG	=	119;		{ set Velocity Gain }
	VO	=	224;		{ set Velocity Override }
	VV	=	172;		{ Vector Velocity }
	XN	=	125;		{ Set XON character }
	XF	=	126;		{ Set XOF character }

{ Motion Commands }
	AB	=	10;		{ ABort }
	AF	=	235;		{ Auxiliary encoder Find index }
	CA	=	180;		{ arc Center Absolute }
	CD	=	193;		{ Contour Distance }
	CM	=	27;		{ set Contouring Mode }
	CP	=	192;		{ Contour Path }
	CR	=	181;		{ arc Center Relative }
	FE	=	11;		{ Find Edge }
	FI	=	12;		{ Find Index }
	GH	=	13;		{ Go Home }
	GM =	 8;		{ Gain Mode }
	GO	=	14;		{ GO }
	HO	=	15;		{ HOme }
	IA	=	93;		{ Index Arm }
	LP	=	112;		{ Learn Position }
	LT	=	113;		{ Learn Target }
	MA	=	16;		{ Move Absolute }
	MF	=	17;		{ Motor ofF }
	MN	=	19;		{ Motor oN }
	MP	=	20;		{ Move to Point }
	MR	=	21;		{ Move Relative }
	NS	=	171;		{ No Syncronization }
	PM	=	23;		{ Position Mode }
	QM	=	 9;		{ Torque Mode }
	SM	=	26;		{ Set Master }
	SN	=	170;		{ Syncroniztion oN }
	ST	=	22;		{ STop }
	VM	=	24;		{ Velocity Mode }

{ Reporting Commands } 
	AT	=	233;		{ Auxiliary encoder Tell position }
	AZ	=	236;		{ Auxiliary encoder tell Index }
	CC	=	70;		{ Current Count }
	CS	=	71;		{ CheckSum }
	HE	=	72;		{ HElp }
	TA	=	73;		{ Tell A/d }
	TB	=	91;		{ Tell Breakpoint Position }
	TC	=	74;		{ Tell Channel }
	TD	=	75;		{ Tell Derivative Gain }
	TF	=	77;		{ Tell Following error }
	TG	=	78;		{ Tell Position Gain }
	TI	=	79;		{ Tell Integral Gain }
	TK	=	92;		{ Tell velocity Konstant }
	TL	=	80;		{ Tell Integration Limit }
	TOO =	89;		{ Tell Optimal }
	TP	=	82;		{ Tell Position }
	TQ	=	209;		{ Tell torQue }
	TS	=	83;		{ Tell Status }
	TT	=	84;		{ Tell Target }
	TV	=	85;		{ Tell Velocity }
	TX	=	88;		{ Tell Contouring Count }
	TZ	=	90;		{ Tell Index }
	VE	=	86;		{ tell VErsion }
	

{ Register Commands }
	AA	=	133;		{ Accumulator Add }
	AC	=	140;		{ Accumulator Complement }
	AD	=	136;		{ Accumulator Divide }
	AE	=	143;		{ Accumulator Eor }
	AL	=	130;		{ Accumulator Load with constant }
	AM	=	135;		{ Accumulator Multiply }
	AN	=	141;		{ Accumulator aNd }
	AO	=	142;		{ Accumulator Or }
	AR	=	132;		{ copy Accumulator to Register }
	AS	=	134;		{ Accumulator Subtract }
	AV	=	139;		{ Accumulator eValuate }
	GA	=	248;		{ Get Analog input reading into accumulator }
	GX	=	247;		{ Get auXiliary encoder position into accumulator }
	OA	=	249;		{ Output Analog value from accumulator }
	RA	=	131;		{ copy Register to Accumulator }
	RB	=	150;		{ Read accumulator Byte }
	RD	=	147;		{ Read accumulator double (64 bit floating point) }
	RL	=	152;		{ Read accumulator Long }
	RV	=	146;		{ Read accumulator float (32 bit floating point) }
	RW	=	151;		{ Read accumulator Word }
	SL	=	144;		{ accumulator Shift Left }
	SR	=	145;		{ accumulator Shift Right }
	TR	=	87;		{ Tell Register }
	WB	=	153;		{ Write accumulator Byte }
	WD	=	149;		{ Write accumulator double (64 bit floating point) }
	WL	=	155;		{ Write accumulator Long }
	WV	=	148;		{ Write accumulator float (32 bit floating point) }
	WW	=	154;		{ Write accumulator Word }

{ Macro Commands }
	ET	=	251;		{ Escape Task }
	GT =	250;		{ Generate Task }
	MC	=	2;			{ Macro Call Command }
	MD	=	3;			{ Macro Definition }
	MJ	=	5;			{ Macro Jump Command }
	RM	=	4;			{ Reset Macros }
	TM	=	81;		{ Tell Macros }

{ Sequence Commands }
	DF	=	107;		{ Do if channel ofF }
	DN	=	106;		{ Do if channel oN }
	IB	=	165;		{ If accumulator Below (less then) }
	IC	=	161;		{ If bit Clear in accumulator }
	IE	=	162;		{ If accumulator Equal }
	IFF =	109;		{ If channel ofF then, else skip two }
	IG	=	164;		{ If accumulator Greater than }
	INN =	108;		{ If channel oN then, else skip two }
	IP	=	96;		{ Interrupt on absolute Position }
	IR	=	97;		{ Interrupt on Relative position }
	IS	=	160;		{ If bit Set in accumulator }
	IU	=	163;		{ If accumulator Unequal }
	JP	=	6;			{ Jump to command absolute }
	JR	=	7;			{ Jump to command relative }
	RP	=	100;		{ RePeat }
	WA	=	101;		{ WAit (time) }
	WE	=	102;		{ Wait Edge }
	WF	=	103;		{ Wait for Channel ofF }
	WI	=	94;		{ Wait for Index }
	WN	=	104;		{ Wait for Channel oN }
	WP	=	98;		{ Wait for absolute Position }
	WR	=	99;		{ Wait for Relative position }
	WS	=	105;		{ Wait for Stop }
	WT	=	198;		{ Wait for Target }

{ Miscellaneous Commands }
	BK	=	121;		{ Break }
	DM	=	60;		{ Decimal Mode }
	HM	=	61;		{ Hex Mode }
	NO	=	120;		{ No Operation }


{ Contour Path Modes }
	USER_DEF	=	0;
	LINE		=	1;
	ARC_CW	=	2;
	ARC_CCW	=	3;


type

{ Type definitions for declaring command buffers (note the different
	parameter types). }
{ No parameter format}
cmdfmt = record
	axcmd : word;
end;

{ Long integer parameter format }
cmdlngfmt = record
	axcmd : word;
	val : longint;
end;

{ Single precision real parameter format }
cmdsglfmt = record
	axcmd : word;
	val : single;
end;

{ Double precision real parameter format }
cmddblfmt = record
	axcmd : word;
	val : double;
end;

cmdbuffmt = array [0..119] of cmdfmt;
cmdlngbuffmt = array [0..39] of cmdlngfmt;
cmdsglbuffmt = array [0..39] of cmdsglfmt;
cmddblbuffmt = array [0..23] of cmddblfmt;

cmdfield = array [0..239] of byte;


{ Type definitions for declaring reply buffers (note the different
	reply types) }
rpyfmt = record
	axcmd : word;
	val : longint;
end;

{ Long integer reply format }
rpylngfmt = record
	axcmd : word;
	val : longint;
end;

{ Single precision real reply format }
rpysglfmt = record
	axcmd : word;
	val : single;
end;

{ Double precision real reply format }
rpydblfmt = record
	axcmd : word;
	val : double;
end;

rpybuffmt = array [0..39] of rpyfmt;
rpylngbuffmt = array [0..39] of rpylngfmt;
rpysglbuffmt = array [0..39] of rpysglfmt;
rpydblbuffmt = array [0..23] of rpydblfmt;

rpyfield = array [0..239] of byte;



{ var }


function dcxrdyin(board:integer) : boolean;
function dcxrdyout(board:integer) : boolean;
function getdcx(board:integer) : byte;
procedure putdcx(board:integer; data:byte);
function dcxbusy(board:integer) : boolean;
procedure dcxvcmd(board:integer; count:integer; var cmdbuf : cmdbuffmt);
procedure dcxcmd(board:integer; count:integer; var cmdbuf : cmdlngbuffmt);
function dcxrpy(board:integer; count:integer; var rpybuf:rpybuffmt):integer;



{ Parameter setup command procedures }
procedure set_acceleration_gain(board,axis:integer;gain:double);
procedure aux_enc_define_home(board,axis:integer);
procedure set_baud_rate(board,baud:integer);
procedure channel_off(board,channel:integer);
procedure channel_high_true_logic(board,channel:integer);
procedure channel_in(board,channel:integer);
procedure channel_low(board,channel:integer);
procedure channel_on(board,channel:integer);
procedure channel_out(board,channel:integer);
procedure set_position_deadband(board,axis:integer;deadband:double);
procedure set_deceleration_gain(board,axis:integer;gain:double);
procedure define_home(board,axis:integer;position:double);
procedure direction(board,axis:integer;dir:integer);
procedure deceleration_set(board,axis:integer;rate:double);
procedure delay_at_target(board,axis:integer;period:double);
procedure full_current(board,axis:integer);
procedure amplifier_fault_input_off(board,axis:integer);
procedure amplifier_fault_input_on(board,axis:integer);
procedure set_derivative_sampling_period(board,axis:integer;period:integer);
procedure half_current(board,axis:integer);
procedure high_motion_limit(board,axis:integer;position:double);
procedure high_speed(board,axis:integer);
procedure set_integration_limit(board,axis:integer;limit:double);
procedure jog_acceleration(board,axis:integer;rate:double);
procedure jog_deadband(board,axis:integer;deadband:double);
procedure jog_proportional_gain(board,axis:integer;gain:double);
procedure jog_offset(board,axis:integer;offset:double);
procedure jog_min_velocity(board,axis:integer;rate:double);
procedure motion_limits_off(board,axis:integer;select:integer);
procedure low_motion_limit(board,axis:integer;position:double);
procedure limit_mode(board,axis:integer;mode:integer);
procedure motion_limits_on(board,axis:integer;select:integer);
procedure low_speed(board,axis:integer);
procedure medium_speed(board,axis:integer);
procedure set_min_velocity(board,axis:integer;rate:double);
procedure set_module_output_mode(board,axis:integer;select:integer);
procedure set_servo_output_phase(board,axis:integer;select:integer);
procedure set_acceleration(board,axis:integer;rate:double);
procedure set_derivative_gain(board,axis:integer;gain:double);
procedure stop_on_error(board,axis:integer;error:double);
procedure step_full(board,axis:integer);
procedure set_proportional_gain(board,axis:integer;gain:double);
procedure step_half(board,axis:integer);
procedure set_integral_gain(board,axis:integer;gain:double);
procedure set_torque(board,axis:integer;torque:double);
procedure set_slave_ratio(board,axis:integer;ratio:double);
procedure set_velocity(board,axis:integer;rate:double);
procedure set_user_output_constant(board,axis:integer;value:double);
procedure set_user_offset(board,axis:integer;value:double);
procedure set_user_rate(board,axis:integer;value:double);
procedure set_user_scale(board,axis:integer;value:double);
procedure set_user_time(board:integer;value:double);
procedure set_user_zero(board,axis:integer;value:double);
procedure vector_acceleration(board,axis:integer;rate:double);
procedure vector_deceleration(board,axis:integer;rate:double);
procedure set_velocity_gain(board,axis:integer;gain:double);
procedure set_velocity_override(board,axis:integer;value:double);
procedure vector_velocity(board,axis:integer;rate:double);


{ Motion command procedures }
procedure abort(board,axis:integer);
procedure aux_enc_find_index(board,axis:integer);
procedure contour_mode(board,axis:integer;caxis:integer);
procedure find_edge(board,axis:integer;position:double);
procedure find_index(board,axis:integer;position:double);
procedure go_home(board,axis:integer);
procedure gain_mode(board,axis:integer);
procedure goo(board,axis:integer);
procedure home(board,axis:integer);
procedure index_arm(board,axis:integer;position:double);
procedure learn_position(board,axis:integer;index:integer);
procedure learn_target(board,axis:integer;index:integer);
procedure move_absolute(board,axis:integer;position:double);
procedure motor_off(board,axis:integer);
procedure motor_on(board,axis:integer);
procedure move_to_point(board,axis:integer;index:integer);
procedure move_relative(board,axis:integer;distance:double);
procedure no_synchronization(board,axis:integer);
procedure position_mode(board,axis:integer);
procedure profile_s_curve(board,axis:integer);
procedure profile_trapezoidal(board,axis:integer);
procedure torque_mode(board,axis:integer);
procedure set_master(board,axis:integer;maxis:integer);
procedure synchronization_on(board,axis:integer);
procedure stop(board,axis:integer);
procedure velocity_mode(board,axis:integer);


{ Reporting command procedures }
procedure tell_aux_enc_pos(board,axis:integer);
procedure tell_aux_enc_idx(board,axis:integer);
procedure tell_analog(board,channel:integer);
procedure tell_breakpoint(board,axis:integer);
procedure tell_channel(board,channel:integer);
procedure tell_derivative_gain(board,axis:integer);
procedure tell_following_error(board,axis:integer);
procedure tell_proportional_gain(board,axis:integer);
procedure tell_integral_gain(board,axis:integer);
procedure tell_velocity_constant(board,axis:integer);
procedure tell_integration_limit(board,axis:integer);
procedure tell_optimal(board,axis:integer);
procedure tell_position(board,axis:integer);
procedure tell_torque(board,axis:integer);
procedure tell_status(board,axis:integer);
procedure tell_target(board,axis:integer);
procedure tell_velocity(board,axis:integer);
procedure tell_contouring_count(board,axis:integer);
procedure tell_index(board,axis:integer);


{ Register command procedures }
procedure accumlator_load_long(board:integer;value:longint);
procedure accumlator_load_float(board:integer;value:single);
procedure accumlator_load_double(board:integer;value:double);
procedure copy_acc_to_reg(board,register:integer);
procedure read_byte(board,axis:integer;location:longint);
procedure read_double(board,axis:integer;location:longint);
procedure read_long(board,axis:integer;location:longint);
procedure read_float(board,axis:integer;location:longint);
procedure read_word(board,axis:integer;location:longint);
procedure copy_reg_to_acc(board,register:integer);
procedure tell_register(board,register:integer);
procedure write_byte(board,axis:integer;location:longint);
procedure write_double(board,axis:integer;location:longint);
procedure write_long(board,axis:integer;location:longint);
procedure write_float(board,axis:integer;location:longint);
procedure write_word(board,axis:integer;location:longint);


{ Macro command procedures }
procedure generate_task(board,macro:integer);
procedure macro_call(board,macro:integer);
procedure macro_jump(board,macro:integer);


{ Sequence command procedures }
procedure interrupt_on_position(board,axis:integer;position:double);
procedure interrupt_on_relative(board,axis:integer;distance:double);
procedure wait(board:integer;period:double);
procedure wait_for_edge(board,axis:integer;level:integer);
procedure wait_for_channel_off(board,channel:integer);
procedure wait_for_index(board,axis:integer);
procedure wait_for_channel_on(board,channel:integer);
procedure wait_for_position(board,axis:integer;position:double);
procedure wait_for_relative(board,axis:integer;distance:double);
procedure wait_for_stop(board,axis:integer;period:double);
procedure wait_for_target(board,axis:integer;period:double);


{ Functions for issuing reporting commands to board and returning values }
function get_aux_enc_pos(board,axis:integer):longint;
function get_aux_enc_idx(board,axis:integer):longint;
function get_analog(board,channel:integer):longint;
function get_breakpoint(board,axis:integer):double;
function get_channel(board,channel:integer):longint;
function get_derivative_gain(board,axis:integer):double;
function get_following_error(board,axis:integer):double;
function get_proportional_gain(board,axis:integer):double;
function get_integral_gain(board,axis:integer):double;
function get_velocity_constant(board,axis:integer):double;
function get_integration_limit(board,axis:integer):double;
function get_optimal(board,axis:integer):double;
function get_position(board,axis:integer):double;
function get_register_long(board,register:integer):longint;
function get_register_float(board,register:integer):single;
function get_register_double(board,register:integer):double;
function get_torque(board,axis:integer):double;
function get_status(board,axis:integer):longint;
function get_target(board,axis:integer):double;
function get_velocity(board,axis:integer):double;
function get_contouring_count(board,axis:integer):longint;
function get_index(board,axis:integer):double;


implementation


{ dcxrdyin() - function returning boolean indicating DCX is ready to accept
								data from PC into the ascii mailbox }
function dcxrdyin(board:integer) : boolean;
begin
	dcxrdyin := byte(ptr(DCXSEG+(board*$100),DATAIN)^) = 0;
end;


{ dcxrdyout() - function returning boolean indicating DCX has data available
								at the ascii mailbox for PC }
function dcxrdyout(board:integer) : boolean;
begin
	dcxrdyout := byte(ptr(DCXSEG+(board*$100),DATAOUT)^) <> 0;
end;


{ putdcx() - Procedure for writing one byte of data to DCX. Use this
				 	procedure to write Ascii commands to DCX input mailbox. }
procedure putdcx(board:integer; data:byte);
begin
	while (not dcxrdyin(board)) do;
	byte(ptr(DCXSEG+(board*$100),DATAIN)^) := data;
end;


{ getdcx() - Function for reading one byte of data from DCX. Use this
					function to read Ascii replies from DCX output mailbox. }
function getdcx(board:integer) : byte;
begin
	while (not dcxrdyout(board)) do;
	getdcx := byte(ptr(DCXSEG+(board*$100),DATAOUT)^);
	byte(ptr(DCXSEG+(board*$100),DATAOUT)^) := 0;
end;


{ dcxbusy() - function returning boolean indicating DCX is busy executing
							peviously sent commands }
function dcxbusy(board:integer) : boolean;
begin
	dcxbusy := byte(ptr(DCXSEG+(board*$100),CMDSTAT)^) <> 0;
end;


{
 dcxvcmd() - Send variable number of commands to a DCX controller using
             binary interface. This function has a parameter specifiying
				 the number of bytes to be passed to the DCX board. Another
				 function in this library, dcxcmd(), has a similar format but
				 has a parameter that specifies the number of commands to be
				 passed to the DCX. That function assumes each command has a
				 4 byte parameter. This does not support the added capability
				 of the series 200 DCX board to accept commands with no
				 parameter or commands with floating point formatted parameters.

 Procedure parameters:
	board = DCX board number (0-15).
	count = number of bytes in DCX commands.
	cmdbuf = buffer containing DCX commands, there can be up to
			255 command bytes passed to function.

  After calling this function, command replies can be read using
  the dcxrpy() function.
}

procedure dcxvcmd(board:integer; count:integer; var cmdbuf : cmdbuffmt);
var
	cmdcntptr : ^byte;
begin

	cmdcntptr := ptr(DCXSEG+(board*$100),CMDCNT);	{ Initialize pointer }

	while (cmdcntptr^ <> 0) do;	{ Wait for DCX to finish previous commands. }

											{ Copy commands to DCX memory }
	move(cmdbuf,ptr(DCXSEG+(board*$100),CMDBLK)^,count);
											{ Set DCX command count to number of bytes
												placed in DCX memory }
   cmdcntptr^ := count;

end;



{
 dcxcmd() - Send one or more commands to a DCX controller through binary
				command buffer. This function has a parameter that specifies
				the number of commands to be passed to the DCX. It assumes
				each command has a 32 bit command parameter. Thus, it does
				not support sending commands with no parameter or commands
				with 64 bit floating point parameters.  This function has
				been replaced by the more flexible version dcxvcmd(), but
				is included in the library for backward compatibility. New
				designs should used the dcxvcmd() if possible.

 Procedure parameters:
	board = DCX board number (0-15).
	count = number of DCX commands.
	cmdbuf = buffer containing DCX commands, there can be up to
			40 commands passed to function.

  After calling this function, command replies can be read using
  the dcxrpy() function.
}

procedure dcxcmd(board:integer; count:integer; var cmdbuf : cmdlngbuffmt);
var
	bytcnt : integer;
	cmdcntptr : ^byte;
begin

	bytcnt := count*6;		{ Determin number of bytes in command block. }

	cmdcntptr := ptr(DCXSEG+(board*$100),CMDCNT);	{ Initialize pointer }

	while (cmdcntptr^ <> 0) do;	{ Wait for DCX to finish previous commands. }

											{ Copy commands to DCX memory }
	move(cmdbuf,ptr(DCXSEG+(board*$100),CMDBLK)^,bytcnt);
											{ Set DCX command count to number of bytes
												placed in DCX memory }
   cmdcntptr^ := bytcnt;

end;



{
 dcxrpy() - Retrieve one or more binary formatted replies from a DCX Motion
 							Controller.

 Function parameters:
	board = DCX board number (0-15).
	count = size of reply buffer
	rpybuf = Buffer where reply from DCX should be placed.

	This function will return all replies from commands currently being
	executed.
}

function dcxrpy(board:integer; count:integer; var rpybuf:rpybuffmt):integer;
var
	bufidx,bytcnt,rpysiz : integer;
	cmdcntptr : ^byte;
	rpycntptr : ^byte;
begin

	bufidx := 0;
	cmdcntptr := ptr(DCXSEG+(board*$100),CMDCNT);
	rpycntptr := ptr(DCXSEG+(board*$100),RPYCNT);

	while (cmdcntptr^ <> 0) do;	{ Wait for DCX to finish current commands. }
										{ If DCX indicates reply ready, read it. }
   bytcnt := rpycntptr^;		{ Get number of bytes in reply buffer }

	if (bytcnt <> 0) then
	begin
		if (count < bytcnt) then		{ Limit reply to size of reply buffer }
			bytcnt := count;

		move(ptr(DCXSEG+(board*$100),RPYBLK)^,rpybuf,bytcnt);

		rpycntptr^ := 0;		{ Set reply count to zero to tell DCX reply read }

		dcxrpy := bytcnt;		{ Return number of bytes in reply }
	end
	else
		dcxrpy := 0;		{ If no reply generated by command return 0. }
end;

{**************************************************************************
	Command procedures
**************************************************************************}

{**************************************************************************
	Parameter setup command procedures
**************************************************************************}

procedure set_acceleration_gain(board,axis:integer;gain:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + AG + DBLPARFLAG;
	cmddblbuf[0].val := gain;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure aux_enc_define_home(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + AH + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure set_baud_rate(board,baud:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := BR;
	cmdlngbuf[0].val := baud;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure channel_off(board,channel:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := CF;
	cmdlngbuf[0].val := channel;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure channel_high_true_logic(board,channel:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := CH;
	cmdlngbuf[0].val := channel;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure channel_in(board,channel:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := CI;
	cmdlngbuf[0].val := channel;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure channel_low(board,channel:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := CL;
	cmdlngbuf[0].val := channel;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure channel_on(board,channel:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := CN;
	cmdlngbuf[0].val := channel;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure channel_out(board,channel:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := CT;
	cmdlngbuf[0].val := channel;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure set_position_deadband(board,axis:integer;deadband:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + DB + DBLPARFLAG;
	cmddblbuf[0].val := deadband;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_deceleration_gain(board,axis:integer;gain:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + DG + DBLPARFLAG;
	cmddblbuf[0].val := gain;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure define_home(board,axis:integer;position:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + DH + DBLPARFLAG;
	cmddblbuf[0].val := position;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure direction(board,axis:integer;dir:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + DI;
	cmdlngbuf[0].val := dir;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure deceleration_set(board,axis:integer;rate:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + DS + DBLPARFLAG;
	cmddblbuf[0].val := rate;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure delay_at_target(board,axis:integer;period:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + DT + DBLPARFLAG;
	cmddblbuf[0].val := period;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure full_current(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + FC + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure amplifier_fault_input_off(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + FF + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure amplifier_fault_input_on(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + FN + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure set_derivative_sampling_period(board,axis:integer;period:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + FR;
	cmdlngbuf[0].val := period;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure half_current(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + HC + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure high_motion_limit(board,axis:integer;position:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + HL + DBLPARFLAG;
	cmddblbuf[0].val := position;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure high_speed(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + HS + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure set_integration_limit(board,axis:integer;limit:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + IL + DBLPARFLAG;
	cmddblbuf[0].val := limit;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure jog_acceleration(board,axis:integer;rate:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + JA + DBLPARFLAG;
	cmddblbuf[0].val := rate;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure jog_deadband(board,axis:integer;deadband:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + DB + DBLPARFLAG;
	cmddblbuf[0].val := deadband;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure jog_proportional_gain(board,axis:integer;gain:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + JG + DBLPARFLAG;
	cmddblbuf[0].val := gain;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure jog_offset(board,axis:integer;offset:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + JO + DBLPARFLAG;
	cmddblbuf[0].val := offset;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure jog_min_velocity(board,axis:integer;rate:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + JV + DBLPARFLAG;
	cmddblbuf[0].val := rate;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure motion_limits_off(board,axis:integer;select:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + LF;
	cmdlngbuf[0].val := select;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure low_motion_limit(board,axis:integer;position:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + LL + DBLPARFLAG;
	cmddblbuf[0].val := position;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure limit_mode(board,axis:integer;mode:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + LM;
	cmdlngbuf[0].val := mode;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure motion_limits_on(board,axis:integer;select:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + LN;
	cmdlngbuf[0].val := select;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure low_speed(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + LS + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure medium_speed(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + MS + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure set_min_velocity(board,axis:integer;rate:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + MV + DBLPARFLAG;
	cmddblbuf[0].val := rate;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_module_output_mode(board,axis:integer;select:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + OM;
	cmdlngbuf[0].val := select;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure set_servo_output_phase(board,axis:integer;select:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + PH;
	cmdlngbuf[0].val := select;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure set_acceleration(board,axis:integer;rate:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + SA + DBLPARFLAG;
	cmddblbuf[0].val := rate;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_derivative_gain(board,axis:integer;gain:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + SD + DBLPARFLAG;
	cmddblbuf[0].val := gain;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure stop_on_error(board,axis:integer;error:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + SE + DBLPARFLAG;
	cmddblbuf[0].val := error;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure step_full(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + SF + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure set_proportional_gain(board,axis:integer;gain:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + SG + DBLPARFLAG;
	cmddblbuf[0].val := gain;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure step_half(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + SH + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure set_integral_gain(board,axis:integer;gain:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + SI + DBLPARFLAG;
	cmddblbuf[0].val := gain;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_torque(board,axis:integer;torque:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + SQ + DBLPARFLAG;
	cmddblbuf[0].val := torque;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_slave_ratio(board,axis:integer;ratio:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + SS + DBLPARFLAG;
	cmddblbuf[0].val := ratio;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_velocity(board,axis:integer;rate:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + SV + DBLPARFLAG;
	cmddblbuf[0].val := rate;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_user_output_constant(board,axis:integer;value:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + UK + DBLPARFLAG;
	cmddblbuf[0].val := value;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_user_offset(board,axis:integer;value:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + UO + DBLPARFLAG;
	cmddblbuf[0].val := value;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_user_rate(board,axis:integer;value:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + UR + DBLPARFLAG;
	cmddblbuf[0].val := value;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_user_scale(board,axis:integer;value:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + US + DBLPARFLAG;
	cmddblbuf[0].val := value;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_user_time(board:integer;value:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := UT + DBLPARFLAG;
	cmddblbuf[0].val := value;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_user_zero(board,axis:integer;value:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + UZ + DBLPARFLAG;
	cmddblbuf[0].val := value;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure vector_acceleration(board,axis:integer;rate:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + VA + DBLPARFLAG;
	cmddblbuf[0].val := rate;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure vector_deceleration(board,axis:integer;rate:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + VD + DBLPARFLAG;
	cmddblbuf[0].val := rate;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_velocity_gain(board,axis:integer;gain:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + VG + DBLPARFLAG;
	cmddblbuf[0].val := gain;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure set_velocity_override(board,axis:integer;value:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + VO + DBLPARFLAG;
	cmddblbuf[0].val := value;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure vector_velocity(board,axis:integer;rate:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + VA + DBLPARFLAG;
	cmddblbuf[0].val := rate;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;



{**************************************************************************
	Motion Command procedures
**************************************************************************}

procedure abort(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + AB + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure aux_enc_find_index(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + AF + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure contour_mode(board,axis:integer;caxis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + CM;
	cmdlngbuf[0].val := caxis;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure find_edge(board,axis:integer;position:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + FE + DBLPARFLAG;
	cmddblbuf[0].val := position;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure find_index(board,axis:integer;position:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + FI + DBLPARFLAG;
	cmddblbuf[0].val := position;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure go_home(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + GH + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure gain_mode(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + GM + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure goo(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + GO + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure home(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + HO + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure index_arm(board,axis:integer;position:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + IA + DBLPARFLAG;
	cmddblbuf[0].val := position;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure learn_position(board,axis:integer;index:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + LP;
	cmdlngbuf[0].val := index;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure learn_target(board,axis:integer;index:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + LT;
	cmdlngbuf[0].val := index;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure move_absolute(board,axis:integer; position:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + MA + DBLPARFLAG;
	cmddblbuf[0].val := position;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure motor_off(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + MF + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure motor_on(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + MN + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure move_to_point(board,axis:integer;index:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + MP;
	cmdlngbuf[0].val := index;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure move_relative(board,axis:integer; distance:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + MR + DBLPARFLAG;
	cmddblbuf[0].val := distance;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure no_synchronization(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + NS + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure position_mode(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + PM + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure profile_s_curve(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + PS + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure profile_trapezoidal(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + PT + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure torque_mode(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + QM + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure set_master(board,axis:integer; maxis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + SM;
	cmdlngbuf[0].val := maxis;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure synchronization_on(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + SN + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure stop(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + ST + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure velocity_mode(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + VM + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;



{**************************************************************************
	Reporting command procedures
**************************************************************************}

procedure tell_aux_enc_pos(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + AT + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure tell_aux_enc_idx(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + AZ + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure tell_analog(board,channel:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := TA;						{ Issue tell command to DCX }
	cmdlngbuf[0].val := channel;					{ Request a long integer reply }
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_breakpoint(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TB;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_channel(board,channel:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := TC;
	cmdlngbuf[0].val := channel;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_derivative_gain(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TD;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_following_error(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TF;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_proportional_gain(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TG;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_integral_gain(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TI;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_velocity_constant(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TK;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_integration_limit(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TL;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_optimal(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TOO;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_position(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TP;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_torque(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TQ;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_status(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + TS + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure tell_target(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TT;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_velocity(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TV;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_contouring_count(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TX;
	cmdlngbuf[0].val := LNGRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_index(board,axis:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + TZ;
	cmdlngbuf[0].val := DBLRPY;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;



{**************************************************************************
   Register command procedures
**************************************************************************}

procedure accumlator_load_long(board:integer;value:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := AL;
	cmdlngbuf[0].val := value;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure accumlator_load_float(board:integer;value:single);
var
	cmdsglbuf:cmdsglbuffmt;
begin
	cmdsglbuf[0].axcmd := AL + SGLPARFLAG;
	cmdsglbuf[0].val := value;
	dcxvcmd(board,SGLCMDSZ,cmdbuffmt(cmdsglbuf));
end;


procedure accumlator_load_double(board:integer;value:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := AL + DBLPARFLAG;
	cmddblbuf[0].val := value;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure copy_acc_to_reg(board,register:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := AR;
	cmdlngbuf[0].val := register;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure read_byte(board,axis:integer;location:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + RB;
	cmdlngbuf[0].val := location;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure read_double(board,axis:integer;location:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + RD;
	cmdlngbuf[0].val := location;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure read_long(board,axis:integer;location:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + RL;
	cmdlngbuf[0].val := location;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure read_float(board,axis:integer;location:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + RV;
	cmdlngbuf[0].val := location;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure read_word(board,axis:integer;location:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + RW;
	cmdlngbuf[0].val := location;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure copy_reg_to_acc(board,register:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := RA;
	cmdlngbuf[0].val := register;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure tell_register(board,register:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := TR;
	cmdlngbuf[0].val := register;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure write_byte(board,axis:integer;location:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + WB;
	cmdlngbuf[0].val := location;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure write_double(board,axis:integer;location:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + WD;
	cmdlngbuf[0].val := location;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure write_long(board,axis:integer;location:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + WL;
	cmdlngbuf[0].val := location;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure write_float(board,axis:integer;location:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + WV;
	cmdlngbuf[0].val := location;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure write_word(board,axis:integer;location:longint);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + WW;
	cmdlngbuf[0].val := location;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;



{**************************************************************************
	Macro command procedures
**************************************************************************}

procedure generate_task(board,macro:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := GT;
	cmdlngbuf[0].val := macro;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure macro_call(board,macro:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := MC;
	cmdlngbuf[0].val := macro;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure macro_jump(board,macro:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := MJ;
	cmdlngbuf[0].val := macro;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;



{**************************************************************************
	Sequence command procedures
**************************************************************************}

procedure interrupt_on_position(board,axis:integer;position:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + IP + DBLPARFLAG;
	cmddblbuf[0].val := position;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure interrupt_on_relative(board,axis:integer;distance:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + IR + DBLPARFLAG;
	cmddblbuf[0].val := distance;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure wait(board:integer;period:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := WA + DBLPARFLAG;
	cmddblbuf[0].val := period;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure wait_for_edge(board,axis:integer;level:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := (axis shl 8) + WE;
	cmdlngbuf[0].val := level;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure wait_for_channel_off(board,channel:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := WF;
	cmdlngbuf[0].val := channel;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure wait_for_index(board,axis:integer);
var
	cmdbuf:cmdbuffmt;
begin
	cmdbuf[0].axcmd := (axis shl 8) + WI + NOPARFLAG;
	dcxvcmd(board,CMDSZ,cmdbuf);
end;


procedure wait_for_channel_on(board,channel:integer);
var
	cmdlngbuf:cmdlngbuffmt;
begin
	cmdlngbuf[0].axcmd := WN;
	cmdlngbuf[0].val := channel;
	dcxvcmd(board,LNGCMDSZ,cmdbuffmt(cmdlngbuf));
end;


procedure wait_for_position(board,axis:integer;position:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + WP + DBLPARFLAG;
	cmddblbuf[0].val := position;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure wait_for_relative(board,axis:integer;distance:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + WR + DBLPARFLAG;
	cmddblbuf[0].val := distance;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure wait_for_stop(board,axis:integer;period:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + WS + DBLPARFLAG;
	cmddblbuf[0].val := period;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;


procedure wait_for_target(board,axis:integer;period:double);
var
	cmddblbuf:cmddblbuffmt;
begin
	cmddblbuf[0].axcmd := (axis shl 8) + WT + DBLPARFLAG;
	cmddblbuf[0].val := period;
	dcxvcmd(board,DBLCMDSZ,cmdbuffmt(cmddblbuf));
end;



{**************************************************************************
	Functions for issuing reporting commands to board and returning values
**************************************************************************}

function get_aux_enc_pos(board,axis:integer):longint;
var
	idx:integer;
	rpylngbuf:rpylngbuffmt;
begin
	tell_aux_enc_pos(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpylngbuf));
   get_aux_enc_pos := rpylngbuf[0].val;
end;


function get_aux_enc_idx(board,axis:integer):longint;
var
	idx:integer;
	rpylngbuf:rpylngbuffmt;
begin
	tell_aux_enc_idx(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpylngbuf));
   get_aux_enc_idx := rpylngbuf[0].val;
end;


function get_analog(board,channel:integer):longint;
var
	idx:integer;
	rpylngbuf:rpylngbuffmt;
begin
	tell_analog(board,channel);						{ Issue tell command to DCX }
   idx := dcxrpy(board,240,rpybuffmt(rpylngbuf));	{ Get reply from DCX }
   get_analog := rpylngbuf[0].val;					{ Copy value from buffer }
end;


function get_breakpoint(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_breakpoint(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_breakpoint := rpydblbuf[0].val;
end;


function get_channel(board,channel:integer):longint;
var
	idx:integer;
	rpylngbuf:rpylngbuffmt;
begin
	tell_channel(board,channel);
   idx := dcxrpy(board,240,rpybuffmt(rpylngbuf));
   get_channel := rpylngbuf[0].val;
end;


function get_derivative_gain(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_derivative_gain(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_derivative_gain := rpydblbuf[0].val;
end;


function get_following_error(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_following_error(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_following_error := rpydblbuf[0].val;
end;


function get_proportional_gain(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_proportional_gain(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_proportional_gain := rpydblbuf[0].val;
end;


function get_integral_gain(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_integral_gain(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_integral_gain := rpydblbuf[0].val;
end;


function get_velocity_constant(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_velocity_constant(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_velocity_constant := rpydblbuf[0].val;
end;


function get_integration_limit(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_integration_limit(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_integration_limit := rpydblbuf[0].val;
end;


function get_optimal(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_optimal(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_optimal := rpydblbuf[0].val;
end;


function get_position(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_position(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_position := rpydblbuf[0].val;
end;


function get_register_long(board,register:integer):longint;
var
	idx:integer;
	rpylngbuf:rpylngbuffmt;
begin
	tell_register(board,register);
   idx := dcxrpy(board,240,rpybuffmt(rpylngbuf));
   get_register_long := rpylngbuf[0].val;
end;


function get_register_float(board,register:integer):single;
var
	idx:integer;
	rpysglbuf:rpysglbuffmt;
begin
	tell_register(board,register);
   idx := dcxrpy(board,240,rpybuffmt(rpysglbuf));
   get_register_float := rpysglbuf[0].val;
end;


function get_register_double(board,register:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_register(board,register);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_register_double := rpydblbuf[0].val;
end;


function get_torque(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_torque(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_torque := rpydblbuf[0].val;
end;


function get_status(board,axis:integer):longint;
var
	idx:integer;
	rpylngbuf:rpylngbuffmt;
begin
	tell_status(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpylngbuf));
   get_status := rpylngbuf[0].val;
end;


function get_target(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_target(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_target := rpydblbuf[0].val;
end;


function get_velocity(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_velocity(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_velocity := rpydblbuf[0].val;
end;


function get_contouring_count(board,axis:integer):longint;
var
	idx:integer;
	rpylngbuf:rpylngbuffmt;
begin
	tell_contouring_count(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpylngbuf));
   get_contouring_count := rpylngbuf[0].val;
end;


function get_index(board,axis:integer):double;
var
	idx:integer;
	rpydblbuf:rpydblbuffmt;
begin
	tell_index(board,axis);
   idx := dcxrpy(board,240,rpybuffmt(rpydblbuf));
   get_index := rpydblbuf[0].val;
end;



end.

